@@ -15,6 +15,7 @@
 from enum import Enum
 from google.cloud.firestore_v1._helpers import decode_value
 import math
+from typing import Any
 
 
 class TypeOrder(Enum):
@@ -31,8 +32,8 @@ class TypeOrder(Enum):
     OBJECT = 9
 
     @staticmethod
-    def from_value(value):
-        v = value.WhichOneof("value_type")
+    def from_value(value) -> Any:
+        v = value._pb.WhichOneof("value_type")
 
         lut = {
             "null_value": TypeOrder.NULL,
@@ -49,7 +50,7 @@ class TypeOrder(Enum):
         }
 
         if v not in lut:
-            raise ValueError("Could not detect value type for " + v)
+            raise ValueError(f"Could not detect value type for {v}")
         return lut[v]
 
 
@@ -59,7 +60,7 @@ class Order(object):
     """
 
     @classmethod
-    def compare(cls, left, right):
+    def compare(cls, left, right) -> int:
         """
         Main comparison function for all Firestore types.
         @return -1 is left < right, 0 if left == right, otherwise 1
@@ -73,7 +74,7 @@ class Order(object):
                 return -1
             return 1
 
-        value_type = left.WhichOneof("value_type")
+        value_type = left._pb.WhichOneof("value_type")
 
         if value_type == "null_value":
             return 0  # nulls are all equal
@@ -98,19 +99,19 @@ class Order(object):
         elif value_type == "map_value":
             return cls.compare_objects(left, right)
         else:
-            raise ValueError("Unknown ``value_type``", str(value_type))
+            raise ValueError(f"Unknown ``value_type`` {value_type}")
 
     @staticmethod
-    def compare_blobs(left, right):
+    def compare_blobs(left, right) -> int:
         left_bytes = left.bytes_value
         right_bytes = right.bytes_value
 
         return Order._compare_to(left_bytes, right_bytes)
 
     @staticmethod
-    def compare_timestamps(left, right):
-        left = left.timestamp_value
-        right = right.timestamp_value
+    def compare_timestamps(left, right) -> Any:
+        left = left._pb.timestamp_value
+        right = right._pb.timestamp_value
 
         seconds = Order._compare_to(left.seconds or 0, right.seconds or 0)
         if seconds != 0:
@@ -119,7 +120,7 @@ class Order(object):
         return Order._compare_to(left.nanos or 0, right.nanos or 0)
 
     @staticmethod
-    def compare_geo_points(left, right):
+    def compare_geo_points(left, right) -> Any:
         left_value = decode_value(left, None)
         right_value = decode_value(right, None)
         cmp = (left_value.latitude > right_value.latitude) - (
@@ -133,7 +134,7 @@ class Order(object):
         )
 
     @staticmethod
-    def compare_resource_paths(left, right):
+    def compare_resource_paths(left, right) -> int:
         left = left.reference_value
         right = right.reference_value
 
@@ -152,7 +153,7 @@ class Order(object):
         return (left_length > right_length) - (left_length < right_length)
 
     @staticmethod
-    def compare_arrays(left, right):
+    def compare_arrays(left, right) -> int:
         l_values = left.array_value.values
         r_values = right.array_value.values
 
@@ -165,7 +166,7 @@ class Order(object):
         return Order._compare_to(len(l_values), len(r_values))
 
     @staticmethod
-    def compare_objects(left, right):
+    def compare_objects(left, right) -> int:
         left_fields = left.map_value.fields
         right_fields = right.map_value.fields
 
@@ -183,13 +184,13 @@ class Order(object):
         return Order._compare_to(len(left_fields), len(right_fields))
 
     @staticmethod
-    def compare_numbers(left, right):
+    def compare_numbers(left, right) -> int:
         left_value = decode_value(left, None)
         right_value = decode_value(right, None)
         return Order.compare_doubles(left_value, right_value)
 
     @staticmethod
-    def compare_doubles(left, right):
+    def compare_doubles(left, right) -> int:
         if math.isnan(left):
             if math.isnan(right):
                 return 0
@@ -200,7 +201,7 @@ class Order(object):
         return Order._compare_to(left, right)
 
     @staticmethod
-    def _compare_to(left, right):
+    def _compare_to(left, right) -> int:
         # We can't just use cmp(left, right) because cmp doesn't exist
         # in Python 3, so this is an equivalent suggested by
         # https://docs.python.org/3.0/whatsnew/3.0.html#ordering-comparisons
