@@ -13,49 +13,49 @@
 # limitations under the License.
 
 import datetime
+import itertools
 import math
 import operator
-import os
-import re
 
 from google.oauth2 import service_account
-from google.protobuf import timestamp_pb2
 import pytest
-import six
 
 from google.api_core.exceptions import AlreadyExists
 from google.api_core.exceptions import FailedPrecondition
 from google.api_core.exceptions import InvalidArgument
 from google.api_core.exceptions import NotFound
-from google.cloud._helpers import _pb_timestamp_to_datetime
+from google.cloud._helpers import _datetime_to_pb_timestamp
 from google.cloud._helpers import UTC
 from google.cloud import firestore_v1 as firestore
-from google.cloud.firestore_v1.client import _FIRESTORE_EMULATOR_HOST
-from test_utils.system import unique_resource_id
-from test_utils.system import EmulatorCreds
 
 from time import sleep
 
-FIRESTORE_CREDS = os.environ.get("FIRESTORE_APPLICATION_CREDENTIALS")
-FIRESTORE_PROJECT = os.environ.get("GCLOUD_PROJECT")
-RANDOM_ID_REGEX = re.compile("^[a-zA-Z0-9]{20}$")
-MISSING_DOCUMENT = "No document to update: "
-DOCUMENT_EXISTS = "Document already exists: "
-UNIQUE_RESOURCE_ID = unique_resource_id("-")
-
-FIRESTORE_EMULATOR = os.getenv(_FIRESTORE_EMULATOR_HOST) is not None
+from tests.system.test__helpers import (
+    FIRESTORE_CREDS,
+    FIRESTORE_PROJECT,
+    RANDOM_ID_REGEX,
+    MISSING_DOCUMENT,
+    UNIQUE_RESOURCE_ID,
+    EMULATOR_CREDS,
+    FIRESTORE_EMULATOR,
+)
 
 
-@pytest.fixture(scope=u"module")
-def client():
+def _get_credentials_and_project():
     if FIRESTORE_EMULATOR:
-        credentials = EmulatorCreds()
+        credentials = EMULATOR_CREDS
         project = FIRESTORE_PROJECT
     else:
         credentials = service_account.Credentials.from_service_account_file(
             FIRESTORE_CREDS
         )
         project = FIRESTORE_PROJECT or credentials.project_id
+    return credentials, project
+
+
+@pytest.fixture(scope="module")
+def client():
+    credentials, project = _get_credentials_and_project()
     yield firestore.Client(project=project, credentials=credentials)
 
 
@@ -76,7 +76,8 @@ def test_collections(client):
 def test_collections_w_import():
     from google.cloud import firestore
 
-    client = firestore.Client()
+    credentials, project = _get_credentials_and_project()
+    client = firestore.Client(project=project, credentials=credentials)
     collections = list(client.collections())
 
     assert isinstance(collections, list)
@@ -97,7 +98,7 @@ def test_create_document(client, cleanup):
         "also": {"nestednow": firestore.SERVER_TIMESTAMP, "quarter": 0.25},
     }
     write_result = document.create(data)
-    updated = _pb_timestamp_to_datetime(write_result.update_time)
+    updated = write_result.update_time
     delta = updated - now
     # Allow a bit of clock skew, but make sure timestamps are close.
     assert -300.0 < delta.total_seconds() < 300.0
@@ -114,7 +115,9 @@ def test_create_document(client, cleanup):
     # NOTE: We could check the ``transform_results`` from the write result
     #       for the document transform, but this value gets dropped. Instead
     #       we make sure the timestamps are close.
-    assert 0.0 <= delta.total_seconds() < 5.0
+    # TODO(microgen): this was 0.0 - 5.0 before. After microgen, This started
+    # getting very small negative times.
+    assert -0.2 <= delta.total_seconds() < 5.0
     expected_data = {
         "now": server_now,
         "eenta-ger": data["eenta-ger"],
@@ -162,9 +165,7 @@ def test_cannot_use_foreign_key(client, cleanup):
 
 
 def assert_timestamp_less(timestamp_pb1, timestamp_pb2):
-    dt_val1 = _pb_timestamp_to_datetime(timestamp_pb1)
-    dt_val2 = _pb_timestamp_to_datetime(timestamp_pb2)
-    assert dt_val1 < dt_val2
+    assert timestamp_pb1 < timestamp_pb2
 
 
 def test_no_document(client):
@@ -354,11 +355,13 @@ def test_update_document(client, cleanup):
         document.update({"bad": "time-past"}, option=option4)
 
     # 6. Call ``update()`` with invalid (in future) "last timestamp" option.
-    timestamp_pb = timestamp_pb2.Timestamp(
-        seconds=snapshot4.update_time.nanos + 3600, nanos=snapshot4.update_time.nanos
-    )
+    timestamp_pb = _datetime_to_pb_timestamp(snapshot4.update_time)
+    timestamp_pb.seconds += 3600
+
     option6 = client.write_option(last_update_time=timestamp_pb)
-    with pytest.raises(FailedPrecondition) as exc_info:
+    # TODO(microgen):invalid argument thrown after microgen.
+    # with pytest.raises(FailedPrecondition) as exc_info:
+    with pytest.raises(InvalidArgument) as exc_info:
         document.update({"bad": "time-future"}, option=option6)
 
 
@@ -387,7 +390,7 @@ def test_document_get(client, cleanup):
         "fire": 199099299,
         "referee": ref_doc,
         "gio": firestore.GeoPoint(45.5, 90.0),
-        "deep": [u"some", b"\xde\xad\xbe\xef"],
+        "deep": ["some", b"\xde\xad\xbe\xef"],
         "map": {"ice": True, "water": None, "vapor": {"deeper": now}},
     }
     write_result = document.create(data)
@@ -404,19 +407,23 @@ def test_document_delete(client, cleanup):
 
     # 1. Call ``delete()`` with invalid (in the past) "last timestamp" option.
     snapshot1 = document.get()
-    timestamp_pb = timestamp_pb2.Timestamp(
-        seconds=snapshot1.update_time.nanos - 3600, nanos=snapshot1.update_time.nanos
-    )
+    timestamp_pb = _datetime_to_pb_timestamp(snapshot1.update_time)
+    timestamp_pb.seconds += 3600
+
     option1 = client.write_option(last_update_time=timestamp_pb)
-    with pytest.raises(FailedPrecondition):
+    # TODO(microgen):invalid argument thrown after microgen.
+    # with pytest.raises(FailedPrecondition):
+    with pytest.raises(InvalidArgument):
         document.delete(option=option1)
 
     # 2. Call ``delete()`` with invalid (in future) "last timestamp" option.
-    timestamp_pb = timestamp_pb2.Timestamp(
-        seconds=snapshot1.update_time.nanos + 3600, nanos=snapshot1.update_time.nanos
-    )
+    timestamp_pb = _datetime_to_pb_timestamp(snapshot1.update_time)
+    timestamp_pb.seconds += 3600
+
     option2 = client.write_option(last_update_time=timestamp_pb)
-    with pytest.raises(FailedPrecondition):
+    # TODO(microgen):invalid argument thrown after microgen.
+    # with pytest.raises(FailedPrecondition):
+    with pytest.raises(InvalidArgument):
         document.delete(option=option2)
 
     # 3. Actually ``delete()`` the document.
@@ -428,6 +435,8 @@ def test_document_delete(client, cleanup):
 
 
 def test_collection_add(client, cleanup):
+    # TODO(microgen): list_documents is returning a generator, not a list.
+    # Consider if this is desired. Also, Document isn't hashable.
     collection_id = "coll-add" + UNIQUE_RESOURCE_ID
     collection1 = client.collection(collection_id)
     collection2 = client.collection(collection_id, "doc", "child")
@@ -522,7 +531,7 @@ def query_docs(client):
     cleanup = []
     stored = {}
     num_vals = 5
-    allowed_vals = six.moves.xrange(num_vals)
+    allowed_vals = range(num_vals)
     for a_val in allowed_vals:
         for b_val in allowed_vals:
             document_data = {
@@ -547,7 +556,7 @@ def test_query_stream_w_simple_field_eq_op(query_docs):
     query = collection.where("a", "==", 1)
     values = {snapshot.id: snapshot.to_dict() for snapshot in query.stream()}
     assert len(values) == len(allowed_vals)
-    for key, value in six.iteritems(values):
+    for key, value in values.items():
         assert stored[key] == value
         assert value["a"] == 1
 
@@ -557,7 +566,7 @@ def test_query_stream_w_simple_field_array_contains_op(query_docs):
     query = collection.where("c", "array_contains", 1)
     values = {snapshot.id: snapshot.to_dict() for snapshot in query.stream()}
     assert len(values) == len(allowed_vals)
-    for key, value in six.iteritems(values):
+    for key, value in values.items():
         assert stored[key] == value
         assert value["a"] == 1
 
@@ -568,18 +577,48 @@ def test_query_stream_w_simple_field_in_op(query_docs):
     query = collection.where("a", "in", [1, num_vals + 100])
     values = {snapshot.id: snapshot.to_dict() for snapshot in query.stream()}
     assert len(values) == len(allowed_vals)
-    for key, value in six.iteritems(values):
+    for key, value in values.items():
         assert stored[key] == value
         assert value["a"] == 1
 
 
+def test_query_stream_w_not_eq_op(query_docs):
+    collection, stored, allowed_vals = query_docs
+    query = collection.where("stats.sum", "!=", 4)
+    values = {snapshot.id: snapshot.to_dict() for snapshot in query.stream()}
+    assert len(values) == 20
+    ab_pairs2 = set()
+    for key, value in values.items():
+        assert stored[key] == value
+        ab_pairs2.add((value["a"], value["b"]))
+
+    expected_ab_pairs = set(
+        [
+            (a_val, b_val)
+            for a_val in allowed_vals
+            for b_val in allowed_vals
+            if a_val + b_val != 4
+        ]
+    )
+    assert expected_ab_pairs == ab_pairs2
+
+
+def test_query_stream_w_simple_not_in_op(query_docs):
+    collection, stored, allowed_vals = query_docs
+    num_vals = len(allowed_vals)
+    query = collection.where("stats.sum", "not-in", [2, num_vals + 100])
+    values = {snapshot.id: snapshot.to_dict() for snapshot in query.stream()}
+
+    assert len(values) == 22
+
+
 def test_query_stream_w_simple_field_array_contains_any_op(query_docs):
     collection, stored, allowed_vals = query_docs
     num_vals = len(allowed_vals)
     query = collection.where("c", "array_contains_any", [1, num_vals * 200])
     values = {snapshot.id: snapshot.to_dict() for snapshot in query.stream()}
     assert len(values) == len(allowed_vals)
-    for key, value in six.iteritems(values):
+    for key, value in values.items():
         assert stored[key] == value
         assert value["a"] == 1
 
@@ -603,7 +642,7 @@ def test_query_stream_w_field_path(query_docs):
     values = {snapshot.id: snapshot.to_dict() for snapshot in query.stream()}
     assert len(values) == 10
     ab_pairs2 = set()
-    for key, value in six.iteritems(values):
+    for key, value in values.items():
         assert stored[key] == value
         ab_pairs2.add((value["a"], value["b"]))
 
@@ -647,7 +686,7 @@ def test_query_stream_w_projection(query_docs):
     query = collection.where("b", "<=", 1).select(["a", "stats.product"])
     values = {snapshot.id: snapshot.to_dict() for snapshot in query.stream()}
     assert len(values) == num_vals * 2  # a ANY, b in (0, 1)
-    for key, value in six.iteritems(values):
+    for key, value in values.items():
         expected = {
             "a": stored[key]["a"],
             "stats": {"product": stored[key]["stats"]["product"]},
@@ -666,7 +705,7 @@ def test_query_stream_w_multiple_filters(query_docs):
         if 5 < a_val * b_val < 10
     ]
     assert len(values) == len(matching_pairs)
-    for key, value in six.iteritems(values):
+    for key, value in values.items():
         assert stored[key] == value
         pair = (value["a"], value["b"])
         assert pair in matching_pairs
@@ -682,14 +721,14 @@ def test_query_stream_w_offset(query_docs):
     #       an ``order_by('a')``, which combined with the ``b == 2``
     #       filter would necessitate an index.
     assert len(values) == num_vals - offset
-    for key, value in six.iteritems(values):
+    for key, value in values.items():
         assert stored[key] == value
         assert value["b"] == 2
 
 
 def test_query_with_order_dot_key(client, cleanup):
     db = client
-    collection_id = "collek" + unique_resource_id("-")
+    collection_id = "collek" + UNIQUE_RESOURCE_ID
     collection = db.collection(collection_id)
     for index in range(100, -1, -1):
         doc = collection.document("test_{:09d}".format(index))
@@ -709,9 +748,9 @@ def test_query_with_order_dot_key(client, cleanup):
         .stream()
     )
     found_data = [
-        {u"count": 30, u"wordcount": {u"page1": 130}},
-        {u"count": 40, u"wordcount": {u"page1": 140}},
-        {u"count": 50, u"wordcount": {u"page1": 150}},
+        {"count": 30, "wordcount": {"page1": 130}},
+        {"count": 40, "wordcount": {"page1": 140}},
+        {"count": 50, "wordcount": {"page1": 150}},
     ]
     assert found_data == [snap.to_dict() for snap in found]
     cursor_with_dotted_paths = {"wordcount.page1": last_value}
@@ -882,6 +921,63 @@ def test_collection_group_queries_filters(client, cleanup):
     assert found == set(["cg-doc2"])
 
 
+def test_partition_query_no_partitions(client, cleanup):
+    collection_group = "b" + UNIQUE_RESOURCE_ID
+
+    # less than minimum partition size
+    doc_paths = [
+        "abc/123/" + collection_group + "/cg-doc1",
+        "abc/123/" + collection_group + "/cg-doc2",
+        collection_group + "/cg-doc3",
+        collection_group + "/cg-doc4",
+        "def/456/" + collection_group + "/cg-doc5",
+    ]
+
+    batch = client.batch()
+    cleanup_batch = client.batch()
+    cleanup(cleanup_batch.commit)
+    for doc_path in doc_paths:
+        doc_ref = client.document(doc_path)
+        batch.set(doc_ref, {"x": 1})
+        cleanup_batch.delete(doc_ref)
+
+    batch.commit()
+
+    query = client.collection_group(collection_group)
+    partitions = list(query.get_partitions(3))
+    streams = [partition.query().stream() for partition in partitions]
+    snapshots = itertools.chain(*streams)
+    found = [snapshot.id for snapshot in snapshots]
+    expected = ["cg-doc1", "cg-doc2", "cg-doc3", "cg-doc4", "cg-doc5"]
+    assert found == expected
+
+
+def test_partition_query(client, cleanup):
+    collection_group = "b" + UNIQUE_RESOURCE_ID
+    n_docs = 128 * 2 + 127  # Minimum partition size is 128
+    parents = itertools.cycle(("", "abc/123/", "def/456/", "ghi/789/"))
+    batch = client.batch()
+    cleanup_batch = client.batch()
+    cleanup(cleanup_batch.commit)
+    expected = []
+    for i, parent in zip(range(n_docs), parents):
+        doc_path = parent + collection_group + f"/cg-doc{i:03d}"
+        doc_ref = client.document(doc_path)
+        batch.set(doc_ref, {"x": i})
+        cleanup_batch.delete(doc_ref)
+        expected.append(doc_path)
+
+    batch.commit()
+
+    query = client.collection_group(collection_group)
+    partitions = list(query.get_partitions(3))
+    streams = [partition.query().stream() for partition in partitions]
+    snapshots = itertools.chain(*streams)
+    found = [snapshot.reference.path for snapshot in snapshots]
+    expected.sort()
+    assert found == expected
+
+
 @pytest.mark.skipif(FIRESTORE_EMULATOR, reason="Internal Issue b/137865992")
 def test_get_all(client, cleanup):
     collection_name = "get-all" + UNIQUE_RESOURCE_ID
@@ -962,7 +1058,7 @@ def test_batch(client, cleanup):
     write_result1 = write_results[0]
     write_result2 = write_results[1]
     write_result3 = write_results[2]
-    assert not write_result3.HasField("update_time")
+    assert not write_result3._pb.HasField("update_time")
 
     snapshot1 = document1.get()
     assert snapshot1.to_dict() == data1
@@ -981,11 +1077,11 @@ def test_batch(client, cleanup):
 
 def test_watch_document(client, cleanup):
     db = client
-    collection_ref = db.collection(u"wd-users" + UNIQUE_RESOURCE_ID)
-    doc_ref = collection_ref.document(u"alovelace")
+    collection_ref = db.collection("wd-users" + UNIQUE_RESOURCE_ID)
+    doc_ref = collection_ref.document("alovelace")
 
     # Initial setting
-    doc_ref.set({u"first": u"Jane", u"last": u"Doe", u"born": 1900})
+    doc_ref.set({"first": "Jane", "last": "Doe", "born": 1900})
     cleanup(doc_ref.delete)
 
     sleep(1)
@@ -999,7 +1095,7 @@ def test_watch_document(client, cleanup):
     doc_ref.on_snapshot(on_snapshot)
 
     # Alter document
-    doc_ref.set({u"first": u"Ada", u"last": u"Lovelace", u"born": 1815})
+    doc_ref.set({"first": "Ada", "last": "Lovelace", "born": 1815})
 
     sleep(1)
 
@@ -1017,11 +1113,11 @@ def test_watch_document(client, cleanup):
 
 def test_watch_collection(client, cleanup):
     db = client
-    collection_ref = db.collection(u"wc-users" + UNIQUE_RESOURCE_ID)
-    doc_ref = collection_ref.document(u"alovelace")
+    collection_ref = db.collection("wc-users" + UNIQUE_RESOURCE_ID)
+    doc_ref = collection_ref.document("alovelace")
 
     # Initial setting
-    doc_ref.set({u"first": u"Jane", u"last": u"Doe", u"born": 1900})
+    doc_ref.set({"first": "Jane", "last": "Doe", "born": 1900})
     cleanup(doc_ref.delete)
 
     # Setup listener
@@ -1038,7 +1134,7 @@ def test_watch_collection(client, cleanup):
     # delay here so initial on_snapshot occurs and isn't combined with set
     sleep(1)
 
-    doc_ref.set({u"first": u"Ada", u"last": u"Lovelace", u"born": 1815})
+    doc_ref.set({"first": "Ada", "last": "Lovelace", "born": 1815})
 
     for _ in range(10):
         if on_snapshot.born == 1815:
@@ -1053,12 +1149,12 @@ def test_watch_collection(client, cleanup):
 
 def test_watch_query(client, cleanup):
     db = client
-    collection_ref = db.collection(u"wq-users" + UNIQUE_RESOURCE_ID)
-    doc_ref = collection_ref.document(u"alovelace")
-    query_ref = collection_ref.where("first", "==", u"Ada")
+    collection_ref = db.collection("wq-users" + UNIQUE_RESOURCE_ID)
+    doc_ref = collection_ref.document("alovelace")
+    query_ref = collection_ref.where("first", "==", "Ada")
 
     # Initial setting
-    doc_ref.set({u"first": u"Jane", u"last": u"Doe", u"born": 1900})
+    doc_ref.set({"first": "Jane", "last": "Doe", "born": 1900})
     cleanup(doc_ref.delete)
 
     sleep(1)
@@ -1068,7 +1164,7 @@ def test_watch_query(client, cleanup):
         on_snapshot.called_count += 1
 
         # A snapshot should return the same thing as if a query ran now.
-        query_ran = collection_ref.where("first", "==", u"Ada").stream()
+        query_ran = collection_ref.where("first", "==", "Ada").stream()
         assert len(docs) == len([i for i in query_ran])
 
     on_snapshot.called_count = 0
@@ -1076,7 +1172,7 @@ def test_watch_query(client, cleanup):
     query_ref.on_snapshot(on_snapshot)
 
     # Alter document
-    doc_ref.set({u"first": u"Ada", u"last": u"Lovelace", u"born": 1815})
+    doc_ref.set({"first": "Ada", "last": "Lovelace", "born": 1815})
 
     for _ in range(10):
         if on_snapshot.called_count == 1:
@@ -1092,14 +1188,14 @@ def test_watch_query(client, cleanup):
 
 def test_watch_query_order(client, cleanup):
     db = client
-    collection_ref = db.collection(u"users")
-    doc_ref1 = collection_ref.document(u"alovelace" + UNIQUE_RESOURCE_ID)
-    doc_ref2 = collection_ref.document(u"asecondlovelace" + UNIQUE_RESOURCE_ID)
-    doc_ref3 = collection_ref.document(u"athirdlovelace" + UNIQUE_RESOURCE_ID)
-    doc_ref4 = collection_ref.document(u"afourthlovelace" + UNIQUE_RESOURCE_ID)
-    doc_ref5 = collection_ref.document(u"afifthlovelace" + UNIQUE_RESOURCE_ID)
+    collection_ref = db.collection("users")
+    doc_ref1 = collection_ref.document("alovelace" + UNIQUE_RESOURCE_ID)
+    doc_ref2 = collection_ref.document("asecondlovelace" + UNIQUE_RESOURCE_ID)
+    doc_ref3 = collection_ref.document("athirdlovelace" + UNIQUE_RESOURCE_ID)
+    doc_ref4 = collection_ref.document("afourthlovelace" + UNIQUE_RESOURCE_ID)
+    doc_ref5 = collection_ref.document("afifthlovelace" + UNIQUE_RESOURCE_ID)
 
-    query_ref = collection_ref.where("first", "==", u"Ada").order_by("last")
+    query_ref = collection_ref.where("first", "==", "Ada").order_by("last")
 
     # Setup listener
     def on_snapshot(docs, changes, read_time):
@@ -1131,19 +1227,19 @@ def test_watch_query_order(client, cleanup):
 
     sleep(1)
 
-    doc_ref1.set({u"first": u"Ada", u"last": u"Lovelace", u"born": 1815})
+    doc_ref1.set({"first": "Ada", "last": "Lovelace", "born": 1815})
     cleanup(doc_ref1.delete)
 
-    doc_ref2.set({u"first": u"Ada", u"last": u"SecondLovelace", u"born": 1815})
+    doc_ref2.set({"first": "Ada", "last": "SecondLovelace", "born": 1815})
     cleanup(doc_ref2.delete)
 
-    doc_ref3.set({u"first": u"Ada", u"last": u"ThirdLovelace", u"born": 1815})
+    doc_ref3.set({"first": "Ada", "last": "ThirdLovelace", "born": 1815})
     cleanup(doc_ref3.delete)
 
-    doc_ref4.set({u"first": u"Ada", u"last": u"FourthLovelace", u"born": 1815})
+    doc_ref4.set({"first": "Ada", "last": "FourthLovelace", "born": 1815})
     cleanup(doc_ref4.delete)
 
-    doc_ref5.set({u"first": u"Ada", u"last": u"lovelace", u"born": 1815})
+    doc_ref5.set({"first": "Ada", "last": "lovelace", "born": 1815})
     cleanup(doc_ref5.delete)
 
     for _ in range(10):
